/*
 * Copyright (c) 1993-94, Silicon Graphics, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the name of Silicon Graphics may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of Silicon Graphics.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL SILICON GRAPHICS BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
 * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
 * POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * OpenGL(TM) is a trademark of Silicon Graphics, Inc.
 */
/*----------------------------------------------------------------------------
 *
 * olight.c : openGL (motif) example showing how to do hardware lighting
 *            including two_sided lighting.
 *
 * Author : Yusuf Attarwala
 *          SGI - Applications
 * Date   : Mar 93
 *
 *    press  left   button for animation
 *           middle button for two sided lighting (default)
 *           right  button for single sided lighting
 *
 *
 *---------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <Xm/Xm.h> 
#include <Xm/Frame.h>               /* for frame widgets */
#include <Xm/Form.h>               /* for frame widgets */
#include <X11/keysym.h>             /* keyboard translations */
#include <X11/StringDefs.h>

#include <GL/gl.h>                  /* gl includes */
#include <GL/glu.h>                 /* utility library includes */
#include <GL/GLwMDrawA.h>           /* include for the drawing area widget */

/* function declarations */

void 
    createToplevel(void),
    drawScene(void),
    setMatrix(void),
    initLightAndMaterial(void),
    animation(void),
    exposeCB(Widget,XtPointer,XtPointer),
    resizeCB(Widget,XtPointer,XtPointer),
    initCB(Widget,XtPointer,XtPointer),
    inputCB(Widget,XtPointer,XtPointer);
 

/* global variables */
            
Display       *display;             /* current display */
XtAppContext  appContext;           /* X application context */
float         ax,ay,az;             /* angles for animation */
GLUquadricObj *quadObj;             /* used in drawscene */
static float lmodel_twoside[] = { GL_TRUE };
static float lmodel_oneside[] = { GL_FALSE };

Widget        toplevel,       /* toplevel shell */
              glw;            /* current widget */
             
GLXContext    glxc;           /* current glx context */

Arg args[20];
int acnt;

void 
main(int argc, char** argv)
{
    XtToolkitInitialize();
    appContext = XtCreateApplicationContext();
    display    = XtOpenDisplay(appContext, NULL, "Olight","olight",NULL,0,
                              &argc,argv);
    if (!display) {
        printf("%s : Unable to open display\n",argv[0]);
        exit(0);
    }

    printf("\n---------------------------------------------\n");
    printf("OpenGL hardware lighting example \n\n");
    printf("Press:  left   button for animation\n\
        middle button for two sided light\n\
        right  button for single sided light\n");

    quadObj = gluNewQuadric ();   /* this will be used in drawScene */
    createToplevel();             /* create widget hierarchy */
    XtAppMainLoop(appContext);    /* loop forever */
}


void
createToplevel(void)
{
    Widget frame;

    acnt = 0;
    XtSetArg(args[acnt],XmNminHeight, 300);acnt++;
    XtSetArg(args[acnt],XmNminWidth,  300);acnt++;
    XtSetArg(args[acnt],XmNminAspectX,  1);acnt++;
    XtSetArg(args[acnt],XmNminAspectY,  1);acnt++;
    XtSetArg(args[acnt],XmNmaxAspectX,  1);acnt++;
    XtSetArg(args[acnt],XmNmaxAspectY,  1);acnt++;
    toplevel  = XtAppCreateShell("openGL light","olight",
                                  applicationShellWidgetClass,
                                  display,args,acnt);

    /* create a frame to hold glx widget */
    frame   = XtVaCreateManagedWidget("frame", 
			      xmFrameWidgetClass, toplevel, 
			      NULL);

    /* create a double buffer widget, in rgb mode and manage it */
    acnt = 0;
    XtSetArg(args[acnt], GLwNrgba,               TRUE); acnt++;
    XtSetArg(args[acnt], GLwNdoublebuffer,       TRUE); acnt++;
    XtSetArg(args[acnt], GLwNallocateBackground, TRUE); acnt++;
    glw = GLwCreateMDrawingArea(frame, "glw", args, acnt);
    XtManageChild(glw);

    /* register callbacks */
    XtAddCallback(glw, GLwNginitCallback,  initCB,   NULL);

    /* realize widget hierarchy */
    XtRealizeWidget(toplevel);

    initLightAndMaterial();

    /* additional callbacks */
    XtAddCallback(glw, GLwNexposeCallback, exposeCB, NULL);
    XtAddCallback(glw, GLwNresizeCallback, resizeCB, NULL);
    XtAddCallback(glw, GLwNinputCallback,  inputCB,  NULL);

}

void
drawScene(void)
{
    glClearColor(0.0, 0.0, 0.0, 0.0);
    glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

    glPushMatrix();
    gluQuadricDrawStyle (quadObj, GLU_FILL);
    glColor3f (1.0, 1.0, 0.0);
    glRotatef (ax,1.0,0.0,0.0);
    glRotatef (-ay,0.0, 1.0, 0.0);

    gluCylinder (quadObj, 2.0,5.0,10.0,20,8);  /* draw a cone */

    glPopMatrix();

    glFlush();
    glXSwapBuffers(XtDisplay(glw), XtWindow(glw));

}

void
setMatrix(void)
{
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho(-15.0,15.0,-15.0,15.0,-10.0,10.0);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
}

void
animation(void)
{
    register int i;
    /* animate the cone */

    for (i=0;i<60;i++) {
        ax += 5.0;
        ay -= 2.0;
        az += 5.0;
        if (ax >= 360)  ax = 0.0;
        if (ay <= -360) ay = 0.0;
        if (az >= 360)  az = 0.0;
        drawScene();
    }
}

void 
inputCB(Widget w, XtPointer client_data, XtPointer call)
{
    char            string[31];
    XComposeStatus  composeStatus;
    KeySym keysym;
    GLwDrawingAreaCallbackStruct *call_data;

    call_data = (GLwDrawingAreaCallbackStruct *) call;

    switch(call_data->event->type) {
    case ButtonPress:
        switch (call_data->event->xbutton.button) {
        case Button1:
	    animation();
            break;
        case Button2 :
            glLightModelfv(GL_LIGHT_MODEL_TWO_SIDE, lmodel_twoside);
	    drawScene();
            break;
        case Button3 :
            glLightModelfv(GL_LIGHT_MODEL_TWO_SIDE, lmodel_oneside);
	    drawScene();
            break;
        }
        break;
    case KeyPress :
        XLookupString((XKeyEvent *)call_data->event,string,
                      30,&keysym,&composeStatus);
        switch(keysym) {
        case XK_Escape :
            exit(0);
            break;
        }
	break;
    default:
        break;
    }
}

void 
resizeCB(Widget w, XtPointer client_data, XtPointer call)
{
    GLwDrawingAreaCallbackStruct *call_data;
    call_data = (GLwDrawingAreaCallbackStruct *) call;


    GLwDrawingAreaMakeCurrent(w, glxc);
    glViewport(0, 0, call_data->width, call_data->height);
    setMatrix();
    drawScene();
}

void 
exposeCB(Widget w, XtPointer client_data, XtPointer call)
{
    GLwDrawingAreaCallbackStruct *call_data;
    call_data = (GLwDrawingAreaCallbackStruct *) call;


    GLwDrawingAreaMakeCurrent(w, glxc);
    glViewport(0, 0, call_data->width, call_data->height);
    drawScene();
}

void
initCB(Widget w, XtPointer client_data, XtPointer call)
{
    XVisualInfo *vi;

    XtSetArg(args[0], GLwNvisualInfo, &vi);
    XtGetValues(w, args, 1);

    glxc = glXCreateContext(XtDisplay(w), vi, 0, GL_TRUE);

    ax = 10.0;
    ay = -10.0;
    az = 0.0;
}

void 
initLightAndMaterial(void)
{
    static float ambient[] = { 0.1, 0.1, 0.1, 1.0 };
    static float diffuse[] = { 0.5, 1.0, 1.0, 1.0 };
    static float position[] = { 90.0, 90.0, 150.0, 0.0 };

    static float front_mat_shininess[] = { 60.0 };
    static float front_mat_specular[] = { 0.2, 0.2, 0.2, 1.0 };
    static float front_mat_diffuse[] = { 0.5, 0.5, 0.28, 1.0 };
    static float back_mat_shininess[] = { 60.0 };
    static float back_mat_specular[] = { 0.5, 0.5, 0.2, 1.0 };
    static float back_mat_diffuse[] = { 1.0, 0.9, 0.2, 1.0 };

    static float lmodel_ambient[] = { 1.0, 1.0, 1.0, 1.0 };

    GLwDrawingAreaMakeCurrent(glw, glxc);

    setMatrix();
    glEnable(GL_DEPTH_TEST);
    glDepthFunc(GL_LEQUAL);

    glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
    glLightfv(GL_LIGHT0, GL_POSITION, position);

    glMaterialfv(GL_FRONT, GL_SHININESS, front_mat_shininess);
    glMaterialfv(GL_FRONT, GL_SPECULAR, front_mat_specular);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, front_mat_diffuse);
    glMaterialfv(GL_BACK, GL_SHININESS, back_mat_shininess);
    glMaterialfv(GL_BACK, GL_SPECULAR, back_mat_specular);
    glMaterialfv(GL_BACK, GL_DIFFUSE, back_mat_diffuse);

    glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
    glLightModelfv(GL_LIGHT_MODEL_TWO_SIDE, lmodel_twoside);

    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);
    glShadeModel(GL_SMOOTH);
}

